﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;

namespace PshaisIndicators
{
    public class TrendWatch : IIndicatorScript
    {
    	int ado_fast_length = 6, ado_slow_length = 26;
    	int ma1_length = 6, ma2_length = 26;
    	Core.MAType ma1_type = Core.MAType.Ema,
    				ma2_type = Core.MAType.Ema;
    	
    	double ado_threshold = 0.0;
    	double ma_threshold = 0.0;
    	double cross_threshold = 0.0;
    	
    	double ado_value, ado_prev_value, ma1_value, ma1_prev_value, ma2_value;
		
        // Properties
        public string Name
        {
            get { return "Pshai's trend watch"; }
        }
        public int TicksBack
        {
            get { 
        		var ma1 = Core.MovingAverageLookback(ma1_length, ma1_type);
        		var ma2 = Core.MovingAverageLookback(ma2_length, ma2_type);
        		var ado = Core.AdOscLookback(ado_fast_length, ado_slow_length);
        		return Max(ma1, ma2, ado) + 5;
        	}
        }
        
        public static int Max(params int[] values)
        {
        	var max = int.MinValue;
        	foreach(var value in values)
        	{
        		max = Math.Max(max, value);
        	}
        	return max;
        }

        // Setting fields
        public List<ScriptParameter> GetParameters()
        {
            List<ScriptParameter> res = new List<ScriptParameter>
            {
                new ScriptParameter("MA Fast Length", ScriptParameterType.Integer, ma1_length.ToString()),
                new ScriptParameter("MA Fast Type", ScriptParameterType.String, ma1_type.ToString()),
                new ScriptParameter("MA Slow Length", ScriptParameterType.Integer, ma2_length.ToString()),
                new ScriptParameter("MA Slow Type", ScriptParameterType.String, ma2_type.ToString()),
                new ScriptParameter("MA Threshold", ScriptParameterType.Decimal, ma_threshold.ToString()),
                new ScriptParameter("AdOsc Fast", ScriptParameterType.Integer, ado_fast_length.ToString()),
                new ScriptParameter("AdOsc Slow", ScriptParameterType.Integer, ado_slow_length.ToString()),
                new ScriptParameter("AdOsc Threshold", ScriptParameterType.Decimal, ado_threshold.ToString()),
                new ScriptParameter("Trade Threshold", ScriptParameterType.Decimal, cross_threshold.ToString())
            };


            return res;
        }

        public void SetParameters(Dictionary<string, object> parameters)
        {
        	ma1_length = Convert.ToInt32(parameters["MA Fast Length"].ToString());
        	ma1_type = ParseMAType(parameters["MA Fast Type"]);
            ma2_length = Convert.ToInt32(parameters["MA Slow Length"].ToString());
            ma2_type = ParseMAType(parameters["MA Slow Type"]);
            ma_threshold = ParseDouble(parameters["MA Threshold"]);
            ado_fast_length = Convert.ToInt32(parameters["AdOsc Fast"].ToString());
            ado_slow_length = Convert.ToInt32(parameters["AdOsc Slow"].ToString());
            ado_threshold = ParseDouble(parameters["AdOsc Threshold"]);
            cross_threshold = ParseDouble(parameters["Trade Threshold"]);
        }

        // Chart lines
        public List<ChartRecord> GetChartLines()
        {
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 0, Name = "MA Fast", HexLineColor = "#FF0000" },
                new ChartRecord() { ChartIndex = 0, Name = "MA Slow", HexLineColor = "#FF8800" },
                new ChartRecord() { ChartIndex = 1, Name = "AdOsc", HexLineColor = "#FF00FF" }
            };
            return chartLines;
        }
        public List<decimal> GetChartData()
        {
            List<decimal> chartData = new List<decimal>
            {
                Convert.ToDecimal(ma1_value),
                Convert.ToDecimal(ma2_value),
                Convert.ToDecimal(ado_value)
            };

            return chartData;
        }

        // This is called as the script is loaded the first time, aka the init
        public void Init()
        {
            // In this code example we do not have to do anything
        }

        // This is the brain so the say, this is where the calculation is done and an output must be produced.
        public IndicatorResult GetResult(IndicatorContext context)
        {
            try
            {
                IPriceInstrument instrument = context.PriceInstrument; // Its pre-created for speed based on the "TicksBack" property field
                int position = instrument.Close.Length - (1 + context.Offset);
                
                var highs = instrument.High;
                var lows = instrument.Low;
                var opens = instrument.Open;
                var closes = instrument.Close;
                var volumes = instrument.Volume;
                
                var median = (highs[position] + lows[position]) / 2;
                
                int ado_begin, ado_len;
                var ado_out = new double[position];
                Core.AdOsc(0, position-1, highs, lows, closes, volumes, ado_fast_length, ado_slow_length, out ado_begin, out ado_len, ado_out);
                
                int ma1_begin, ma1_len;
                var ma1_out = new double[position];
                Core.MovingAverage(0, position-1, closes, ma1_length, ma1_type, out ma1_begin, out ma1_len, ma1_out);
                
                int ma2_begin, ma2_len;
                var ma2_out = new double[position];
                Core.MovingAverage(0, position-1, closes, ma2_length, ma2_type, out ma2_begin, out ma2_len, ma2_out);
                
                
                if(ado_len > 0 && ado_out.Length > 0
                   && ma1_len > 0 && ma1_out.Length > 0
                   && ma2_len > 0 && ma2_out.Length > 0)
                {
                	ado_value = ado_out[ado_len-1];
                	ado_prev_value = ado_out[ado_len-2];
                	ma1_value = ma1_out[ma1_len-1];
                	ma1_prev_value = ma1_out[ma1_len-2];
                	ma2_value = ma2_out[ma2_len-1];
                	
                	if((median < ma1_value * (1 + cross_threshold)
                	   && median > ma1_value * (1 - cross_threshold))
                	   && (ma1_value < ma2_value * (1 + cross_threshold)
                	       && ma1_value > ma2_value * (1 - cross_threshold)) )
                	   return IndicatorResult.StayNoPosition;
                	
                	if( ma1_value * (1 - ma_threshold) > ma2_value && ado_value * (1 - ado_threshold) > ado_prev_value )
                	{
                		return IndicatorResult.BuyLong;
                	}
                	
                	if( ma1_value < ma2_value || (ma1_value * (1 + ma_threshold) < ma1_prev_value && ado_value * (1 + ado_threshold) < ado_prev_value ))
                	{
                		return IndicatorResult.SellShort;
                	}
                }

            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex.Message));
            }

            return IndicatorResult.StayNoPosition; // Default output
        }
        
        public static Core.MAType ParseMAType(object value)
		{
			Core.MAType val;
			switch( value.ToString().ToLower() )
			{
				case "sma":
					val = Core.MAType.Sma;
					break;
               	case "ema":
					val = Core.MAType.Ema;
					break;
               	case "dema":
					val = Core.MAType.Dema;
					break;
               	case "tema":
					val = Core.MAType.Tema;
					break;
               	case "trima":
					val = Core.MAType.Trima;
					break;
               	case "t3":
					val = Core.MAType.T3;
					break;
               	case "mama":
					val = Core.MAType.Mama;
					break;
               	case "kama":
					val = Core.MAType.Kama;
					break;
				case "wma":
					val = Core.MAType.Wma;
					break;
				default:
					val = Core.MAType.Sma;
					break;
			}
			
			return val;
		}
        
        double ParseDouble(object obj)
		{
			var numStyle = System.Globalization.NumberStyles.Any;
			var cult = System.Globalization.CultureInfo.InvariantCulture;
			
			double output;
			try {
				double.TryParse(
					obj.ToString().Replace(',', '.'),
					numStyle,
					cult,
					out output);
			}
			catch
			{
				output = 0;
			}
			
			return output;
		}
    }
}
